@@ -37,6 +37,7 @@ struct CFCFunction;
 struct CFCMethod;
 struct CFCVariable;
 struct CFCFileSpec;
+struct CFCJson;
 
 /** Return true if the string is a valid class name.
  */
@@ -63,11 +64,10 @@ CFCClass_validate_class_name_component(const char *name);
  * to this class.
  * @param file_spec - Clownfish::CFC::Model::FileSpec of the file in which
  * this class was declared
- * @param docucomment A Clownfish::CFC::Model::DocuComment describing this
- * Class.
+ * @param parent_class_name The name of the parent class.
+ * @param is_final Should be true if the class is final.
  * @param is_inert Should be true if the class is inert, i.e. cannot be
  * instantiated.
- * @param is_final Should be true if the class is final.
  * @param is_abstract Should be true if the class is abstract.
  */
 CFCClass*
@@ -89,15 +89,13 @@ CFCClass_destroy(CFCClass *self);
 
 /** Retrieve a Class, if one has already been created.
  *
- * @param A Clownfish::CFC::Model::Parcel.
  * @param class_name The name of the Class.
  */
 CFCClass*
-CFCClass_fetch_singleton(struct CFCParcel *parcel, const char *class_name);
+CFCClass_fetch_singleton(const char *class_name);
 
 /** Retrieve a Class by its struct sym.
  *
- * @param A Clownfish::CFC::Model::Parcel.
  * @param full_struct_sym The Class's full struct sym.
  */
 CFCClass*
@@ -170,6 +168,12 @@ CFCClass_grow_tree(CFCClass *self);
 CFCClass**
 CFCClass_tree_to_ladder(CFCClass *self);
 
+/** Read host-specific data for the class from a JSON hash.
+ */
+void
+CFCClass_read_host_data_json(CFCClass *self, struct CFCJson *hash,
+                            const char *path);
+
 /** Return an array of all methods implemented in this class.
  * Must not be freed by the caller.
  */
