@@ -15,7 +15,6 @@
  */
 
 #include <string.h>
-#include <ctype.h>
 #include <stdio.h>
 
 #define CFC_NEED_CALLABLE_STRUCT_DEF
@@ -27,6 +26,7 @@
 #include "CFCParamList.h"
 #include "CFCDocuComment.h"
 #include "CFCVariable.h"
+#include "CFCJson.h"
 
 #ifndef true
     #define true 1
@@ -66,15 +66,15 @@ S_validate_meth_name(const char *meth_name) {
     int need_upper  = true;
     int need_letter = true;
     for (;; meth_name++) {
-        if (need_upper  && !isupper(*meth_name)) { return false; }
-        if (need_letter && !isalpha(*meth_name)) { return false; }
+        if (need_upper  && !CFCUtil_isupper(*meth_name)) { return false; }
+        if (need_letter && !CFCUtil_isalpha(*meth_name)) { return false; }
         need_upper  = false;
         need_letter = false;
 
         // We've reached NULL-termination without problems, so succeed.
         if (!*meth_name) { return true; }
 
-        if (!isalnum(*meth_name)) {
+        if (!CFCUtil_isalnum(*meth_name)) {
             if (*meth_name != '_') { return false; }
             need_upper  = true;
         }
@@ -252,6 +252,35 @@ CFCMethod_can_be_bound(CFCMethod *method) {
            && CFCCallable_can_be_bound((CFCCallable*)method);
 }
 
+void
+CFCMethod_read_host_data_json(CFCMethod *self, CFCJson *hash,
+                              const char *path) {
+    int         excluded = false;
+    const char *alias    = NULL;
+
+    CFCJson **children = CFCJson_get_children(hash);
+    for (int i = 0; children[i]; i += 2) {
+        const char *key = CFCJson_get_string(children[i]);
+
+        if (strcmp(key, "excluded") == 0) {
+            excluded = CFCJson_get_bool(children[i+1]);
+        }
+        else if (strcmp(key, "alias") == 0) {
+            alias = CFCJson_get_string(children[i+1]);
+        }
+        else {
+            CFCUtil_die("Unexpected key '%s' in '%s'", key, path);
+        }
+    }
+
+    if (excluded) {
+        CFCMethod_exclude_from_host(self);
+    }
+    else if (alias) {
+        CFCMethod_set_host_alias(self, alias);
+    }
+}
+
 void
 CFCMethod_set_host_alias(CFCMethod *self, const char *alias) {
     if (!alias || !alias[0]) {
