@@ -17,6 +17,7 @@
 #include "charmony.h"
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
 /* For rmdir */
@@ -36,11 +37,9 @@
 #include "CFCTest.h"
 #include "CFCUtil.h"
 
-#define T_CFBASE          "t" CHY_DIR_SEP "cfbase"
-#define T_CFEXT           "t" CHY_DIR_SEP "cfext"
-#define T_CFDEST          "t" CHY_DIR_SEP "cfdest"
-#define T_CFDEST_INCLUDE  T_CFDEST CHY_DIR_SEP "include"
-#define T_CFDEST_SOURCE   T_CFDEST CHY_DIR_SEP "source"
+#define AUTOGEN          "autogen"
+#define AUTOGEN_INCLUDE  AUTOGEN CHY_DIR_SEP "include"
+#define AUTOGEN_SOURCE   AUTOGEN CHY_DIR_SEP "source"
 
 static void
 S_run_tests(CFCTest *test);
@@ -51,9 +50,12 @@ S_run_basic_tests(CFCTest *test);
 static void
 S_run_include_tests(CFCTest *test);
 
+static void
+S_run_clash_tests(CFCTest *test);
+
 const CFCTestBatch CFCTEST_BATCH_HIERARCHY = {
     "Clownfish::CFC::Model::Hierarchy",
-    44,
+    41,
     S_run_tests
 };
 
@@ -61,20 +63,23 @@ static void
 S_run_tests(CFCTest *test) {
     S_run_basic_tests(test);
     S_run_include_tests(test);
+    S_run_clash_tests(test);
 }
 
 static void
 S_run_basic_tests(CFCTest *test) {
-    CFCHierarchy *hierarchy = CFCHierarchy_new(T_CFDEST);
-    STR_EQ(test, CFCHierarchy_get_dest(hierarchy), T_CFDEST, "get_dest");
-    STR_EQ(test, CFCHierarchy_get_include_dest(hierarchy), T_CFDEST_INCLUDE,
+    char *cfbase_path = CFCTest_path("cfbase");
+
+    CFCHierarchy *hierarchy = CFCHierarchy_new(AUTOGEN);
+    STR_EQ(test, CFCHierarchy_get_dest(hierarchy), AUTOGEN, "get_dest");
+    STR_EQ(test, CFCHierarchy_get_include_dest(hierarchy), AUTOGEN_INCLUDE,
            "get_include_dest");
-    STR_EQ(test, CFCHierarchy_get_source_dest(hierarchy), T_CFDEST_SOURCE,
+    STR_EQ(test, CFCHierarchy_get_source_dest(hierarchy), AUTOGEN_SOURCE,
            "get_source_dest");
 
-    CFCHierarchy_add_source_dir(hierarchy, T_CFBASE);
+    CFCHierarchy_add_source_dir(hierarchy, cfbase_path);
     const char **source_dirs = CFCHierarchy_get_source_dirs(hierarchy);
-    STR_EQ(test, source_dirs[0], T_CFBASE, "source_dirs[0]");
+    STR_EQ(test, source_dirs[0], cfbase_path, "source_dirs[0]");
     OK(test, source_dirs[1] == NULL, "source_dirs[1]");
 
     CFCHierarchy_build(hierarchy);
@@ -123,12 +128,10 @@ S_run_basic_tests(CFCTest *test) {
     time_t now       = time(NULL);
     time_t past_time = now - 2;
     static const char *const h_paths[] = {
-        T_CFDEST_INCLUDE CHY_DIR_SEP "Animal.h",
-        T_CFDEST_INCLUDE CHY_DIR_SEP "Animal" CHY_DIR_SEP "Dog.h",
-        T_CFDEST_INCLUDE CHY_DIR_SEP "Animal" CHY_DIR_SEP "Util.h"
+        AUTOGEN_INCLUDE CHY_DIR_SEP "Animal.h",
+        AUTOGEN_INCLUDE CHY_DIR_SEP "Animal" CHY_DIR_SEP "Dog.h",
+        AUTOGEN_INCLUDE CHY_DIR_SEP "Animal" CHY_DIR_SEP "Util.h"
     };
-    OK(test, CFCUtil_make_path(T_CFDEST_INCLUDE CHY_DIR_SEP "Animal"),
-       "make_path");
     for (int i = 0; i < 3; ++i) {
         const char *h_path  = h_paths[i];
         const char *content = "#include <stdio.h>\n";
@@ -136,9 +139,8 @@ S_run_basic_tests(CFCTest *test) {
         CFCTest_set_file_times(h_path, past_time);
     }
 
-    char *cfh_path = CFCFile_cfh_path(animal, T_CFBASE);
+    const char *cfh_path = CFCFile_get_path(animal);
     CFCTest_set_file_times(cfh_path, now);
-    FREEMEM(cfh_path);
 
     CFCHierarchy_propagate_modified(hierarchy, 0);
 
@@ -151,24 +153,29 @@ S_run_basic_tests(CFCTest *test) {
     for (int i = 0; i < 3; ++i) {
         remove(h_paths[i]);
     }
-    rmdir(T_CFDEST_INCLUDE CHY_DIR_SEP "Animal");
-    rmdir(T_CFDEST_INCLUDE);
-    rmdir(T_CFDEST_SOURCE);
-    rmdir(T_CFDEST);
+    rmdir(AUTOGEN_INCLUDE CHY_DIR_SEP "Animal");
+    rmdir(AUTOGEN_INCLUDE);
+    rmdir(AUTOGEN_SOURCE);
+    rmdir(AUTOGEN);
 
     CFCBase_decref((CFCBase*)hierarchy);
+    FREEMEM(cfbase_path);
     CFCClass_clear_registry();
     CFCParcel_reap_singletons();
 }
 
 static void
 S_run_include_tests(CFCTest *test) {
+    char *cfbase_path = CFCTest_path("cfbase");
+    char *cfext_path  = CFCTest_path("cfext");
+    char *cfinc_path  = CFCTest_path("cfinc");
+
     {
-        CFCHierarchy *hierarchy = CFCHierarchy_new(T_CFDEST);
-        CFCHierarchy_add_source_dir(hierarchy, T_CFEXT);
-        CFCHierarchy_add_include_dir(hierarchy, T_CFBASE);
+        CFCHierarchy *hierarchy = CFCHierarchy_new(AUTOGEN);
+        CFCHierarchy_add_source_dir(hierarchy, cfext_path);
+        CFCHierarchy_add_include_dir(hierarchy, cfinc_path);
         const char **include_dirs = CFCHierarchy_get_include_dirs(hierarchy);
-        STR_EQ(test, include_dirs[0], T_CFBASE, "include_dirs[0]");
+        STR_EQ(test, include_dirs[0], cfinc_path, "include_dirs[0]");
         OK(test, include_dirs[1] == NULL, "include_dirs[1]");
 
         CFCHierarchy_build(hierarchy);
@@ -201,9 +208,9 @@ S_run_include_tests(CFCTest *test) {
     }
 
     {
-        CFCHierarchy *hierarchy = CFCHierarchy_new(T_CFDEST);
-        CFCHierarchy_add_source_dir(hierarchy, T_CFBASE);
-        CFCHierarchy_add_source_dir(hierarchy, T_CFEXT);
+        CFCHierarchy *hierarchy = CFCHierarchy_new(AUTOGEN);
+        CFCHierarchy_add_source_dir(hierarchy, cfbase_path);
+        CFCHierarchy_add_source_dir(hierarchy, cfext_path);
 
         CFCHierarchy_build(hierarchy);
 
@@ -229,37 +236,96 @@ S_run_include_tests(CFCTest *test) {
         CFCParcel_reap_singletons();
     }
 
-    {
-        CFCHierarchy *hierarchy = CFCHierarchy_new(T_CFDEST);
-        CFCHierarchy_add_include_dir(hierarchy, T_CFBASE);
-        CFCHierarchy_add_include_dir(hierarchy, T_CFEXT);
-        CFCHierarchy_add_prereq(hierarchy, "AnimalExtension");
+    rmdir(AUTOGEN_INCLUDE);
+    rmdir(AUTOGEN_SOURCE);
+    rmdir(AUTOGEN);
 
-        CFCHierarchy_build(hierarchy);
+    FREEMEM(cfbase_path);
+    FREEMEM(cfext_path);
+    FREEMEM(cfinc_path);
+}
 
-        CFCParcel *animal = CFCParcel_fetch("Animal");
-        OK(test, animal != NULL, "parcel Animal registered");
-        OK(test, CFCParcel_required(animal), "parcel Animal required");
-        CFCParcel *animal_ext = CFCParcel_fetch("AnimalExtension");
-        OK(test, animal_ext != NULL, "parcel AnimalExtension registered");
-        OK(test, CFCParcel_required(animal_ext),
-           "parcel AnimalExtension required");
+static void
+S_run_clash_tests(CFCTest *test) {
+    char *cfbase_path        = CFCTest_path("cfbase");
+    char *cfinc_path         = CFCTest_path("cfinc");
+    char *cfclash_file_path  = CFCTest_path("cfclash" CHY_DIR_SEP "file");
+    char *cfclash_class_path = CFCTest_path("cfclash" CHY_DIR_SEP "class");
+    char *cfclash_bar_path   = CFCTest_path("cfclash" CHY_DIR_SEP "bar");
 
-        CFCClass **classes = CFCHierarchy_ordered_classes(hierarchy);
-        int num_classes = 0;
-        while (classes[num_classes]) {
-            ++num_classes;
+    if (getenv("CLOWNFISH_VALGRIND")) {
+        SKIP(test, 1, "Exceptions leak");
+    }
+    else {
+        CFCHierarchy *hierarchy = CFCHierarchy_new(AUTOGEN);
+        CFCHierarchy_add_source_dir(hierarchy, cfbase_path);
+        CFCHierarchy_add_source_dir(hierarchy, cfclash_file_path);
+        char *error;
+
+        CFCUTIL_TRY {
+            CFCHierarchy_build(hierarchy);
         }
-        INT_EQ(test, num_classes, 5, "class count");
+        CFCUTIL_CATCH(error);
+        OK(test, error && strstr(error, "found twice"),
+           "source/source filename clash");
 
-        FREEMEM(classes);
         CFCBase_decref((CFCBase*)hierarchy);
         CFCClass_clear_registry();
         CFCParcel_reap_singletons();
     }
 
-    rmdir(T_CFDEST_INCLUDE);
-    rmdir(T_CFDEST_SOURCE);
-    rmdir(T_CFDEST);
+    if (getenv("CLOWNFISH_VALGRIND")) {
+        SKIP(test, 1, "Exceptions leak");
+    }
+    else {
+        CFCHierarchy *hierarchy = CFCHierarchy_new(AUTOGEN);
+        CFCHierarchy_add_source_dir(hierarchy, cfclash_class_path);
+        CFCHierarchy_add_include_dir(hierarchy, cfinc_path);
+        char *error;
+
+        CFCUTIL_TRY {
+            CFCHierarchy_build(hierarchy);
+        }
+        CFCUTIL_CATCH(error);
+        OK(test, error && strstr(error, "Two classes with name"),
+           "source/include class name clash");
+
+        CFCBase_decref((CFCBase*)hierarchy);
+        CFCClass_clear_registry();
+        CFCParcel_reap_singletons();
+    }
+
+    if (getenv("CLOWNFISH_VALGRIND")) {
+        SKIP(test, 1, "Exceptions leak");
+    }
+    else {
+        CFCHierarchy *hierarchy = CFCHierarchy_new(AUTOGEN);
+        CFCHierarchy_add_source_dir(hierarchy, cfclash_bar_path);
+        CFCHierarchy_add_include_dir(hierarchy, cfinc_path);
+        char *error;
+
+        CFCUTIL_TRY {
+            CFCHierarchy_build(hierarchy);
+        }
+        CFCUTIL_CATCH(error);
+        const char *message =
+            "Class Baz from source dir found in parcel Foo from include dir";
+        OK(test, error && strstr(error, message),
+           "source class with included parcel");
+
+        CFCBase_decref((CFCBase*)hierarchy);
+        CFCClass_clear_registry();
+        CFCParcel_reap_singletons();
+    }
+
+    rmdir(AUTOGEN_INCLUDE);
+    rmdir(AUTOGEN_SOURCE);
+    rmdir(AUTOGEN);
+
+    FREEMEM(cfbase_path);
+    FREEMEM(cfinc_path);
+    FREEMEM(cfclash_file_path);
+    FREEMEM(cfclash_class_path);
+    FREEMEM(cfclash_bar_path);
 }
 
