@@ -27,7 +27,20 @@
 extern "C" {
 #endif
 
+#include <stdarg.h>
 #include <stddef.h>
+#include <setjmp.h>
+
+#define CFCUTIL_TRY                                   \
+    do {                                              \
+        jmp_buf env;                                  \
+        jmp_buf *prev_env = CFCUtil_try_start(&env);  \
+        if (!setjmp(env))
+
+#define CFCUTIL_CATCH(error)                          \
+        error = CFCUtil_try_end(prev_env);            \
+    } while (0)
+
 
 /** Create an inner Perl object with a refcount of 1.  For use in actual
  * Perl-space, it is necessary to wrap this inner object in an RV.
@@ -58,6 +71,9 @@ CFCUtil_strndup(const char *string, size_t len);
 char*
 CFCUtil_sprintf(const char *fmt, ...);
 
+char*
+CFCUtil_vsprintf(const char *fmt, va_list args);
+
 /** Concatenate a NULL-terminated list of strings onto the first, reallocating
  * with each argument.
  */
@@ -137,6 +153,33 @@ CFCUtil_wrapped_free(void *ptr);
 #define FREEMEM(_ptr) \
     CFCUtil_wrapped_free(_ptr)
 
+/** Safe wrappers for ctype.h functions.
+ */
+
+int
+CFCUtil_isalnum(char c);
+
+int
+CFCUtil_isalpha(char c);
+
+int
+CFCUtil_isdigit(char c);
+
+int
+CFCUtil_islower(char c);
+
+int
+CFCUtil_isspace(char c);
+
+int
+CFCUtil_isupper(char c);
+
+char
+CFCUtil_tolower(char c);
+
+char
+CFCUtil_toupper(char c);
+
 /** Given two filepaths, return true if the second exists and has a
  * modification time which more recent than that of the first.
  */
@@ -219,11 +262,22 @@ CFCUtil_free_string_array(char **strings);
 void
 CFCUtil_die(const char *format, ...);
 
+/* Rethrow an error.
+ */
+void
+CFCUtil_rethrow(char *error);
+
 /* Print an error message to stderr.
  */
 void
 CFCUtil_warn(const char *format, ...);
 
+jmp_buf*
+CFCUtil_try_start(jmp_buf *env);
+
+char*
+CFCUtil_try_end(jmp_buf *prev_env);
+
 #ifdef __cplusplus
 }
 #endif
